#include <u.h>
#include <libc.h>
#include <json.h>
#include "jenkins.h"

 
static int
buildscan(Node *n, Sess *s)
{
	Fmt f;
	int i, j;
	Node *cc;
	Jvalue *v;
	long mtime, elap, expected;
	char *p, *uid, *job, *bld, buf[1024];

	bld = n->dir.name;
	job = n->parent->dir.name;
	if((v = exch(s, nil, "job/%s/%s/api/json", job, bld)) == nil){
		warn("cannot get build info, %r\n");
		return -1;
	}

	mtime = 0;
	uid = nil;

	if((p = json_strval(v, "timestamp")) != nil)
		mtime = strtoll(p, nil, 0)/1000;

	snprint(buf, sizeof(buf), "job/%s/%s/consoleText", job, bld);
	newurl(n, "log", nil, nil, nil, mtime, 0, buf);

	if((p = json_strval(v, "actions[0]/causes[0]/shortDescription")) != nil)
		newfile(n, "cause", nil, nil, nil, mtime, mtime, p);

	if((p = json_strval(v, "result")) != nil)
		newfile(n, "result", nil, nil, nil, mtime, mtime, strlwr(p));

	if((p = json_strval(v, "building")) != nil)
		newfile(n, "building", nil, nil, nil, mtime, mtime, p);

	snprint(buf, sizeof(buf), "job/%s/ws/*zip*/%s.zip", job, job);
	newurl(n, "workspace.zip", nil, nil, nil, mtime, 0, buf);

	fmtstrinit(&f);
	for(i = 0; (p = json_strval(v, "culprits[%d]/fullName", i)) != nil; i++)
		fmtprint(&f, "%s\n", p);
	p = fmtstrflush(&f);
	newfile(n, "culprits", nil, nil, nil, mtime, mtime, p);
	free(p);

	if((p = json_strval(v, "duration")) != nil){
		expected = 0;
		elap = strtol(p, nil, 0);
		if((p = json_strval(v, "estimatedDuration")) != nil)
			expected = strtol(p, nil, 0);
		snprint(buf, sizeof(buf), "elapsed: %ld.%-03.3ld sec\nexpected: %ld.%-03.3ld sec\n",
			elap/60000, elap%60000, expected/60000, expected%60000);
		newfile(n, "duration", nil, nil, nil, mtime, mtime, buf);
	}

	cc = newdir(n, nil, nil, nil, "artifacts");
	for(i = 0; (p = json_strval(v, "artifacts[%d]/relativePath", i)) != nil; i++){
		snprint(buf, sizeof(buf), "job/%s/%s/artifact/%s", job, bld, p);

		if((p = json_strval(v, "artifacts[%d]/fileName", i)) != nil)
			newurl(cc, p, uid, uid, uid, mtime, 0, buf);
	}

	for(i = 0; (p = json_strval(v, "changeSet[%d]/commitId", i)) != nil; i++){
		snprint(buf, sizeof(buf), "r%s", p);
		cc = newdir(n, uid, nil, nil, buf);

		if((p = json_strval(v, "changeSet[%d]/user", i)) != nil)
			uid = p;
		if((p = json_strval(v, "changeSet[%d]/timestamp", i)) != nil)
			mtime = strtoll(p, nil, 0)/1000;

		fmtstrinit(&f);
		for(j = 0; (p = json_strval(v, "changeSet[%d]/affectedPaths[%d]", i, j)) != nil; j++)
			fmtprint(&f, "%s\n", p);
		p = fmtstrflush(&f);
		newfile(cc, "files", uid, uid, uid, mtime, mtime, p);
		free(p);

		if((p = json_strval(v, "changeSet[%d]/msg", i)) != nil)
			newfile(cc, "logmsg", uid, uid, uid, mtime, mtime, p);
	}

	json_free(v);
	return 0;
}

/*
 * We expect to get nothing back from a start-build request (null Json pointer).
 * we free it in case future versions of the server tell us anything.
 */
static int
startbuild(Node *n, Sess *s, void *, int)
{
	Jvalue *v;
	char *job;

	job = n->parent->dir.name;
	if((v = exch(s, nil, "job/%s/build?token=%s&cause=Manual+trigger", job, s->buildtoken)) == nil)
		return -1;
	json_free(v);
	return 0;
}


static int
jobinfo(Node *n, Sess *s)
{
	int i;
	Fmt f;
	Node *cc;
	Jvalue *v;
	char *p, *job;
	static char *builds[] = { 
		"lastBuild", "lastCompletedBuild", "lastFailedBuild",
		"lastSuccessfulBuild", "lastUnstableBuild"
	};

	job = n->dir.name;
	if((v = exch(s, nil, "job/%s/api/json", job)) == nil){
		warn("%s cannot get job info, %r\n", job);
		return -1;
	}

	if((p = json_strval(v, "description")) != nil)
		newfile(n, "description", nil, nil, nil, 0, 0, p);

	if((p = json_strval(v, "healthReport[0]/score")) != nil)
		newfile(n, "health", nil, nil, nil, 0, 0, p);

	if((p = json_strval(v, "buildable")) != nil)
		newfile(n, "buildable", nil, nil, nil, 0, 0, p);

	fmtstrinit(&f);
	for(i = 0; (p = json_strval(v, "downstreamProjects[%d]/name", i)) != nil; i++)
		fmtprint(&f, "%s\n", p);
	p = fmtstrflush(&f);
	newfile(n, "dependencies", nil, nil, nil, 0, 0, p);
	free(p);

	if((p = json_strval(v, "inQueue")) != nil){
		cc = newfile(n, "queued", nil, nil, nil, 0, 0, p);
		if(s->buildtoken){
			cc->flush = startbuild;
			writable(cc);
		}
	}

	for(i = 0; i < nelem(builds); i++){
		if(json_strval(v, "%s/number", builds[i]) != nil){
			cc = newdir(n, nil, nil, nil, builds[i]);
			cc->scan = buildscan;
		}
	}

	for(i = 0; (p = json_strval(v, "builds[%d]/number", i)) != nil; i++){
		cc = newdir(n, nil, nil, nil, p);
		cc->scan = buildscan;
	}

	json_free(v);
	return 0;
}

int
jobwalk(Node *n, Sess *s, char *name)
{
	Node *ch;
	Jvalue *v;

	if((v = exch(s, nil, "job/%s/api/json", name)) == nil)
		return -1;
	ch = newdir(n, nil, nil, nil, name);
	ch->scan = jobscan;

	json_free(v);
	return 0;
}

int
jobscan(Node *n, Sess *s)
{
	int i;
	Node *ch;
	Jvalue *v;
	char *name;

	if((v = exch(s, nil, "api/json")) == nil){
		warn("cannot get job list, %r\n");
		return -1;
	}

	for(i = 0; (name = json_strval(v, "jobs[%d]/name", i)) != nil; i++){
		ch = newdir(n, name, nil, nil, name);
		ch->scan = jobinfo;
	}

	json_free(v);
	return 0;
}

int
serverscan(Node *n, Sess *s)
{
	int i;
	char *p;
	Node *ch;
	Jvalue *v;

	if((v = exch(s, nil, "computer/api/json")) == nil){
		warn("cannot scan for users, %r\n");
		return -1;
	}
	for(i = 0; (p = json_strval(v, "computer[%d]/displayName", i)) != nil; i++){
		ch = newdir(n, nil, nil, nil, p);
		if((p = json_strval(v, "computer[%d]/idle")) != nil)
			newfile(ch, "idle", nil, nil, nil, 0, 0, p);

		if((p = json_strval(v, "computer[%d]/monitorData/hudson.node_monitors.SwapSpaceMonitor/availablePhysicalMemory")) != nil)
			newfile(ch, "memfree", nil, nil, nil, 0, 0, p);
		if((p = json_strval(v, "computer[%d]/monitorData/hudson.node_monitors.SwapSpaceMonitor/totalPhysicalMemory")) != nil)
			newfile(ch, "memtotal", nil, nil, nil, 0, 0, p);

		if((p = json_strval(v, "computer[%d]/monitorData/hudson.node_monitors.SwapSpaceMonitor/availableSwapSpace")) != nil)
			newfile(ch, "swapfree", nil, nil, nil, 0, 0, p);
		if((p = json_strval(v, "computer[%d]/monitorData/hudson.node_monitors.SwapSpaceMonitor/totalSwapSpace")) != nil)
			newfile(ch, "swaptotal", nil, nil, nil, 0, 0, p);
		
		if((p = json_strval(v, "computer[%d]/monitorData/hudson.node_monitors.ArchitectureMonitor")) != nil)
			newfile(ch, "arch", nil, nil, nil, 0, 0, p);

		if((p = json_strval(v, "computer[%d]/monitorData/hudson.node_monitors.DiskSpaceMonitor/size")) != nil)
			newfile(ch, "cwdsize", nil, nil, nil, 0, 0, p);

		if((p = json_strval(v, "computer[%d]/monitorData/hudson.node_monitors.TemporarySpaceMonitor/size")) != nil)
			newfile(ch, "tmpsize", nil, nil, nil, 0, 0, p);
		
		if((p = json_strval(v, "computer[%d]/numExecutors")) != nil)
			newfile(ch, "threads", nil, nil, nil, 0, 0, p);

		if((p = json_strval(v, "computer[%d]/offline")) != nil)
			newfile(ch, "offline", nil, nil, nil, 0, 0, p);

	}
	json_free(v);
	return 0;
}

int
hostscan(Node *n, Sess *s)
{
	char *p;
	Jvalue *v;

	if((v = exch(s, nil, "api/json")) == nil){
		warn("cannot get host info, %r\n");
		return -1;
	}
	if((p = json_strval(v, "nodeDescription")) != nil)
		newfile(n, "description", nil, nil, nil, 0, 0, p);
	if((p = json_strval(v, "nodeName")) != nil)
		newfile(n, "name", nil, nil, nil, 0, 0, p);
	if((p = json_strval(v, "numExecutors")) != nil)
		newfile(n, "threads", nil, nil, nil, 0, 0, p);
	newfile(n, "version", nil, nil, nil, 0, 0, s->version);

	json_free(v);
	return 0;
}


int
usrwalk(Node *n, Sess *s, char *name)
{
	int i;
	char *p;
	Node *ch;
	Jvalue *v;

	if((v = exch(s, nil, "user/%s/api/json", name)) == nil)
		return -1;

	ch = newdir(n, nil, nil, nil, name);
	if((p = json_strval(v, "description")) != nil)
		newfile(ch, "discription", name, nil, nil, 0, 0, p);
	if((p = json_strval(v, "fullName")) != nil)
		newfile(ch, "fullname", name, nil, nil, 0, 0, p);
	for(i = 0; i < 10; i++)
		if((p = json_strval(v, "property[%d]/address", i)) != nil){
			newfile(ch, "email", name, nil, nil, 0, 0, p);
			break;
		}

	json_free(v);
	return 0;
}

int
usrscan(Node *n, Sess *s)
{
	int i;
	char *p;
	Node *ch;
	Jvalue *v;

	if((v = exch(s, nil, "people/api/json")) == nil){
		warn("cannot scan for users, %r\n");
		return -1;
	}
	for(i = 0; (p = json_strval(v, "users[%d]/user/absoluteUrl", i)) != nil; i++){
		ch = newdir(n, nil, nil, nil, basename(p));
		ch->walk = usrwalk;
	}

	json_free(v);
	return 0;
}


